@@ -1,8 +1,8 @@
 #!/bin/sh
 
 CT_ROOT=$1
-CT_VSN=1.9
-TS_VSN=3.7.2
+CT_VSN=1.10
+TS_VSN=3.8
 
 if [ -z "$CT_ROOT" ]
 then
